@@ -46,6 +46,9 @@
 #define MIN_SIZE (480)
 #define MAX_SIZE (1024)
 
+/* Would be great, but leads to segfaults for some odd reason */
+/* #define USE_FULLSCREEN 0 */
+
 typedef struct {
 	OssoABookMenuExtension parent_instance;
 } QRcodeMenuExtension;
@@ -64,7 +67,7 @@ typedef struct _PluginData PluginData;
 struct _PluginData {
 	/* UI */
 	GtkWindow *window;
-	GtkWidget *dr, *menu, *btn_save, *btn_load, *btn_share;
+	GtkWidget *dr, *menu, *btn_save, *btn_load, *btn_share, *btn_fs;
 
 	/* Current */
 	OssoABookMenuExtension *ext;
@@ -198,17 +201,28 @@ g_return_val_if_fail(pd, FALSE);
 cr=gdk_cairo_create(widget->window);
 gdk_cairo_rectangle(cr, &event->area);
 
-if (pd->cs) {
-	gdouble ratio;
-	gint sh;
+cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
+cairo_paint(cr);
 
+if (pd->cs) {
+	gdouble ratio, x;
+	gint sh, sw;
+
+	sw=cairo_image_surface_get_width(pd->cs);
 	sh=cairo_image_surface_get_height(pd->cs);
+
 	ratio=(gdouble)widget->allocation.height/(gdouble)sh;
-	g_debug("R: %f", ratio);
+
+	x=(widget->allocation.width/2.0)-((sw*ratio)/2.0);
+
+	g_debug("SW: %d SH: %d", sw, sh);
+	g_debug("R: %f X: %f", ratio, x);
+
+	cairo_translate(cr, x, 0.0);
+
 	cairo_scale(cr, ratio, ratio);
-	cairo_set_source_surface(cr, pd->cs, 0, 0);
-} else
-	cairo_set_source_rgb(cr, 1, 1, 1);
+	cairo_set_source_surface(cr, pd->cs, 0.0, 0.0);
+}
 
 cairo_fill(cr);
 cairo_destroy(cr);
@@ -216,6 +230,28 @@ cairo_destroy(cr);
 return FALSE;
 }
 
+#ifdef USE_FULLSCREEN
+static void
+ac_fs_cb(GtkWidget *widget, PluginData *pd)
+{
+g_return_if_fail(pd);
+g_return_if_fail(GTK_IS_WINDOW(pd->window));
+
+gtk_window_fullscreen(pd->window);
+}
+#endif
+
+#ifdef USE_FULLSCREEN
+static void
+ac_unfs_cb(GtkWidget *widget, PluginData *pd)
+{
+g_return_if_fail(pd);
+g_return_if_fail(GTK_IS_WINDOW(pd->window));
+
+gtk_window_unfullscreen(window);
+}
+#endif
+
 /**
  * hildon_app_menu_button_new:
  *
@@ -270,14 +306,22 @@ if (pd->name)
 	gtk_window_set_title(GTK_WINDOW(pd->window), pd->name);
 
 pd->dr=gtk_drawing_area_new();
-gtk_container_add(GTK_CONTAINER(pd->window), GTK_WIDGET(pd->dr));
-g_signal_connect(pd->dr, "expose-event", G_CALLBACK(ac_draw_expose_event), pd);
 
 menu=HILDON_APP_MENU(hildon_app_menu_new());
 pd->btn_save=hildon_app_menu_button_new(menu, pd->contact ? TRUE : FALSE, "Save", G_CALLBACK(ac_qrcode_save_cb), pd);
 pd->btn_load=hildon_app_menu_button_new(menu, pd->contact ? FALSE : TRUE, "Load", G_CALLBACK(ac_qrcode_load_cb), pd);
 pd->btn_share=hildon_app_menu_button_new(menu, pd->contact ? TRUE : FALSE, "Share", G_CALLBACK(ac_qrcode_share_cb), pd);
 
+#ifdef USE_FULLSCREEN
+pd->btn_fs=hildon_app_menu_button_new(menu, TRUE, "Fullscreen", G_CALLBACK(ac_fs_cb), pd);
+gtk_widget_add_events(pd->dr, GDK_BUTTON_RELEASE_MASK);
+g_signal_connect(pd->dr, "button_release_event", G_CALLBACK(ac_unfs_cb), pd);
+#endif
+
+gtk_container_add(GTK_CONTAINER(pd->window), GTK_WIDGET(pd->dr));
+g_signal_connect(pd->dr, "expose-event", G_CALLBACK(ac_draw_expose_event), pd);
+
+
 hildon_window_set_app_menu(HILDON_WINDOW(pd->window), menu);
 
 gtk_widget_show_all(GTK_WIDGET(pd->window));
